package lrs.visitor.test;

import junit.framework.TestCase;
import lrs.*;
import lrs.visitor.*;
/**
 * A JUnit test case class.
 * Every method starting with the word "test" will be called when running
 * the test with JUnit.
 */
public class Test_Zipper extends TestCase {
  
  
  public void test_Zipper() {
    IAlgo algo = Zipper.Singleton;
    
    LRStruct lrs1 = new LRStruct();
    LRStruct lrs2 = new LRStruct();
    lrs1.execute(algo, lrs2);
    
    assertEquals("2 empty lists", "()", lrs1.toString());
    assertEquals("2 empty lists", "()", lrs2.toString());
    
    
    lrs1.insertFront("a");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("1 empty lists", "(a)", lrs1.toString());
    assertEquals("1 empty lists", "(a)", lrs2.toString());
    
    lrs1 = new LRStruct();
    
    lrs1.execute(algo, lrs2);    
    assertEquals("1 empty lists", "(a)", lrs1.toString());
    assertEquals("1 empty lists", "(a)", lrs2.toString());
    
    
    lrs1 = new LRStruct();
    lrs1.insertFront("b");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("2 1-element lists", "(b a)", lrs1.toString());
    assertEquals("2 1-element lists", "(b a)", lrs2.toString());
    
    lrs1 = new LRStruct();
    lrs1.insertFront("c");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("1 and 2 element lists", "(c b a)", lrs1.toString());
    assertEquals("1 and 2 element lists", "(c b a)", lrs2.toString());
    
    lrs1 = new LRStruct();
    lrs1.insertFront("b");
    lrs1.insertFront("a");
    
    lrs2 = new LRStruct();
    lrs2.insertFront("c");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("2 and 1 element lists", "(a c b)", lrs1.toString());
    assertEquals("2 and 1 element lists", "(a c b)", lrs2.toString());
    
    lrs1 = new LRStruct();
    lrs1.insertFront("b");
    lrs1.insertFront("a");
    
    lrs2 = new LRStruct();
    lrs2.insertFront("d");
    lrs2.insertFront("c");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("2 and 2 element lists", "(a c b d)", lrs1.toString());
    assertEquals("2 and 2 element lists", "(a c b d)", lrs2.toString());

    // Test to make sure there is no sharing
    lrs1 = (new LRStruct()).insertFront("a");
    lrs2 = (new LRStruct()).insertFront("b");
    lrs1.execute(algo, lrs2);
    
    try {
      lrs1.execute(algo, lrs2);
      //The counting code should throw an error or exception
      int n1 = (Integer)lrs1.execute(CountElements.Singleton);
      int n2 = (Integer)lrs1.execute(CountElements.Singleton);;
      fail("Sharing test should have failed!");
    }
    catch(Error err){
      // Expected outcome
      System.out.println("Expected sharing test failure with error: "+err);
    }
    catch(Exception e) {
      // Expected outcome
      System.out.println("Expected sharing test failure with exception: "+e);
    }
    
    
  }
  
  public void test_BetterZipper() {
    IAlgo algo = BetterZipper.Singleton;
    
    LRStruct lrs1 = new LRStruct();
    LRStruct lrs2 = new LRStruct();
    lrs1.execute(algo, lrs2);
    
    assertEquals("2 empty lists", "()", lrs1.toString());
    assertEquals("2 empty lists", "()", lrs2.toString());
    
    
    lrs1.insertFront("a");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("1 empty lists", "(a)", lrs1.toString());
    assertEquals("1 empty lists", "(a)", lrs2.toString());
    
    lrs1 = new LRStruct();
    
    lrs1.execute(algo, lrs2);    
    assertEquals("1 empty lists", "(a)", lrs1.toString());
    assertEquals("1 empty lists", "(a)", lrs2.toString());
    
    
    lrs1 = new LRStruct();
    lrs1.insertFront("b");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("2 1-element lists", "(b a)", lrs1.toString());
    assertEquals("2 1-element lists", "(b a)", lrs2.toString());
    
    lrs1 = new LRStruct();
    lrs1.insertFront("c");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("1 and 2 element lists", "(c b a)", lrs1.toString());
    assertEquals("1 and 2 element lists", "(c b a)", lrs2.toString());
    
    lrs1 = new LRStruct();
    lrs1.insertFront("b");
    lrs1.insertFront("a");
    
    lrs2 = new LRStruct();
    lrs2.insertFront("c");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("2 and 1 element lists", "(a c b)", lrs1.toString());
    assertEquals("2 and 1 element lists", "(a c b)", lrs2.toString());
    
    lrs1 = new LRStruct();
    lrs1.insertFront("b");
    lrs1.insertFront("a");
    
    lrs2 = new LRStruct();
    lrs2.insertFront("d");
    lrs2.insertFront("c");
    
    lrs1.execute(algo, lrs2);    
    assertEquals("2 and 2 element lists", "(a c b d)", lrs1.toString());
    assertEquals("2 and 2 element lists", "(a c b d)", lrs2.toString());
    
    // Test to make sure there is no sharing
    lrs1 = (new LRStruct()).insertFront("a");
    lrs2 = (new LRStruct()).insertFront("b");
    lrs1.execute(algo, lrs2);
    
    try {
      lrs1.execute(algo, lrs2);
      //The counting code should be fine now
      int n1 = (Integer)lrs1.execute(CountElements.Singleton);
      int n2 = (Integer)lrs1.execute(CountElements.Singleton);
      //The toString code should be fine now
      System.out.println("lrs1 = "+lrs1 +"\nlrs2 = "+lrs2);
    }
    catch(Error err){
      fail("Expected sharing test failure with error: "+err);
    }
    catch(Exception e) {
      fail("Expected sharing test failure with exception: "+e);
    }
  }

 
}
