package elec;

/**
 * Represents a switch with one input wire and two output terminals.
 * When the switch is UP, the input wire is connected to the UP output terminal.
 * When the switch is DOWN, the input wire is connected to the DOWN output terminal.
 */
public class BetterOne2TwoSwitch implements ITwoWaySwitch{
  
  // STUDENT TO ADD NESTED AND/OR INNER/ANONYMOUS CLASSES
  // TO IMPLEMENT STATE DESIGN PATTERN
  
  /**
   * Abstract  state of the switch
   */
  private static abstract class AOne2TwoSwitchState {
    
    /**
     * Gets the output voltage on the UP terminal
     * @param wire  The input wire to the switch
     */
    abstract int getVoltageUp();
    
    /**
     * Gets the output voltage on the DOWN terminal
     * @param wire  The input wire to the switch
     */
    abstract int getVoltageDown();
  }
  
  /**
   * UP state of the switch
   */
  private AOne2TwoSwitchState _upState = new AOne2TwoSwitchState() {
    /**
     * returns the voltage on the input wire
     */
    int getVoltageUp() {
      return _inputWire.getVoltage();
    }
    
    /**
     * Returns 0 volts, always (disconnected)
     */
    int getVoltageDown() {
      return 0;
    }
  };
  
  /**
   * DOWN state of the switch
   */
  private AOne2TwoSwitchState _downState = new AOne2TwoSwitchState() {
    /**
     * returns the voltage on the input wire
     */
    int getVoltageUp() {
      return 0;
    }
    
    /**
     * Returns 0 volts, always (disconnected)
     */
    int getVoltageDown() {
      return _inputWire.getVoltage();
    }
  };
  
  /**
   * The input wire to the switch
   */
  private IWire _inputWire;
  /**
   * The current state of the switch
   * STUDENT TO COMPLETE INITIALIZATION OF STATE
   */
  private AOne2TwoSwitchState state = _upState;
  
  /**
   * The constructor for the class
   * @param wire The input wire to the switch
   */
  public BetterOne2TwoSwitch(IWire wire) {
    _inputWire = wire;
  }
  
  /**
   * Set the switch to UP
   */
  public void setUp() {
    // STUDENT TO COMPLETE
        
    state = _upState;
  }
  
  /**
   * Set the switch to DOWN
   */
  public void setDown() {
    // STUDENT TO COMPLETE
    
    state = _downState;
  }
  
  /**
   * Factory method for a wire that returns 
   * the voltage on the UP Terminal
   * @return An IWire object
   */
  public IWire getVoltageUpWire() {
    // STUDENT TO COMPLETE
    
    return new IWire() {
      public int getVoltage() {
        return state.getVoltageUp();
      }
    };
  }
  
  /**
   * Factory method for a wire that returns 
   * the voltage on the DOWN Terminal
   * @return An IWire object
   */
  public IWire getVoltageDownWire() {
    // STUDENT TO COMPLETE
    
    return new IWire() {
      public int getVoltage() {
        return state.getVoltageDown();
      }
    };
  }
  
}




