package elec;

/**
 * Represents a switch that has two input wires and one output wire
 * When the switch is UP, the wireUp is connected to the output wire.
 * When the switch is DOWN, the wireDown is connected to the output wire.
 */
public class BetterTwo2OneSwitch implements ITwoWaySwitch, IWire{
  /**
   * The UP input wire
   */
  private IWire _wireUp;
  /**
   * The DOWN input wire
   */
  private IWire _wireDown;
  
  // STUDENT TO ADD NESTED AND/OR INNER/ANONYMOUS CLASSES
  // TO IMPLEMENT STATE DESIGN PATTERN

  /**
   * Abstract  state of the switch
   */
  private static abstract class ATwo2OneSwitchState {
    /**
     * Returns the voltage on the output wire of the switch
     * @param wireUp   The UP input wire
     * @param wireDown  The DOWN input wire
     */
    abstract int getVoltage();
  }
  
  /**
   * UP state of the switch
   */
  private ATwo2OneSwitchState _upState = new ATwo2OneSwitchState() {
    /**
     * Returns the voltage on the UP input wire
     */
    int getVoltage() {
      return _wireUp.getVoltage();
    }
  };
  
  /**
   * DOWN state of the switch
   */
  private ATwo2OneSwitchState _downState = new ATwo2OneSwitchState() {
    /**
     * Returns the voltage on the UP input wire
     */
    int getVoltage() {
      return _wireDown.getVoltage();
    }
  };
  
  /**
   * The current state of switch
   * STUDENT TO COMPLETE INITIALIZATION OF STATE
   */
  private ATwo2OneSwitchState _state = _upState;
  
  /**
   * Constructor for the class
   * @param wireUp  The wire that is connected when the switch is UP.
   * @param wireDown The wire that is connecte when the switch is DOWN.
   */
  public BetterTwo2OneSwitch(IWire wireUp, IWire wireDown) {
    _wireUp = wireUp;
    _wireDown = wireDown;
  }
  
  /**
   * Set the switch to UP
   */
  public void setUp() {
    // STUDENT TO COMPLETE
    _state = _upState;
  }
  
  /**
   * Set the switch to DOWN
   */
  public void setDown() {
    // STUDENT TO COMPLETE
    _state = _downState;
  }
  
  /**
   * Get the voltage on the output wire
   */
  public int getVoltage() {
    return _state.getVoltage();
  }
}


