package elec.test;
import elec.*;
import junit.framework.TestCase;


public class Test_ThreeWaySwitch extends TestCase {
 
  /**
   * An input test voltage for testWire1
   */
  private int testVoltage1 = 0;
  

  /**
   * An input test voltage for testWire2
   */
  private int testVoltage2 = 0;
  
  /**
   * A special wire that returns the voltage value set 
   * by testVoltage1
   */
  private IWire testWire1 = new IWire() {
    public int getVoltage() {
      return testVoltage1;
    }
  };
  
  /** 
   * A special wire that returns the voltage value set 
   * by testVoltage2
   */
  private IWire testWire2 = new IWire() {
    public int getVoltage() {
      return testVoltage2;
    }
  };
  
  
  public void test_Two2OneSwitch() {

    testVoltage1 = 10;
    testVoltage2 = -10;

    Two2OneSwitch sw = new Two2OneSwitch(testWire1, testWire2);

    assertEquals("initial -> +10V", 10, sw.getVoltage());

    sw.setDown();
    assertEquals("down -> -10V", -10, sw.getVoltage());
        
    sw.setUp();
    assertEquals("up -> +10V", 10, sw.getVoltage());

    testVoltage1 = 42;
    assertEquals("up -> +42V", 42, sw.getVoltage());
    
    testVoltage2 = -99;
    assertEquals("up -> +42V", 42, sw.getVoltage());
    
    sw.setDown();
    assertEquals("down -> -99V", -99, sw.getVoltage());
  }
  

  public void test_One2TwoSwitch() {
    testVoltage1 = 10;
    One2TwoSwitch sw = new One2TwoSwitch(testWire1);
    
    IWire upWire = new SwitchUpWire(sw);
    IWire downWire = new SwitchDownWire(sw);
    
    assertEquals("up:  up wire --> 10)", 10, upWire.getVoltage());
    assertEquals("up:  down wire --> 0)", 0, downWire.getVoltage());
    
    sw.setDown();
    assertEquals("down:  up wire --> 0)", 0, upWire.getVoltage());
    assertEquals("down:  down wire --> 10)", 10, downWire.getVoltage());
    
    testVoltage1 = 42;
    assertEquals("down:  up wire --> 0)", 0, upWire.getVoltage());
    assertEquals("down:  down wire --> 42)", 42, downWire.getVoltage());
    
    sw.setUp();
    assertEquals("up:  up wire --> 42)", 42, upWire.getVoltage());
    assertEquals("up:  down wire --> 0)", 0, downWire.getVoltage());
    
  }
  
  
  
  
  public void test_ThreeWaySwitch() {
    
    testVoltage1 = 120;
    
    ThreeWaySwitch sw = new ThreeWaySwitch(testWire1);
    
    assertEquals("up/up --> 120", 120, sw.getVoltage());
    
    sw.setSwitchADown();
    assertEquals("down/up --> 0", 0, sw.getVoltage());
    
    sw.setSwitchAUp();
    assertEquals("up/up --> 120", 120, sw.getVoltage());
    
    sw.setSwitchBDown();
    assertEquals("up/down --> 0", 0, sw.getVoltage());
    
    sw.setSwitchBUp();
    assertEquals("up/up --> 120", 120, sw.getVoltage());
    
    sw.setSwitchADown();
    sw.setSwitchBDown();
    assertEquals("down/down --> 120", 120, sw.getVoltage());
    
    testVoltage1 = -99;
    assertEquals("down/down --> -99", -99, sw.getVoltage());
    
    sw.setSwitchAUp();
    assertEquals("up/down --> 0", 0, sw.getVoltage());
    
    sw.setSwitchBUp();
    assertEquals("up/up --> -99", -99, sw.getVoltage());
    
  }
  
  
  public void test_BetterTwo2OneSwitch() {

    testVoltage1 = 10;
    testVoltage2 = -10;

    BetterTwo2OneSwitch sw = new BetterTwo2OneSwitch(testWire1, testWire2);

    assertEquals("initial -> +10V", 10, sw.getVoltage());

    sw.setDown();
    assertEquals("down -> -10V", -10, sw.getVoltage());
        
    sw.setUp();
    assertEquals("up -> +10V", 10, sw.getVoltage());

    testVoltage1 = 42;
    assertEquals("up -> +42V", 42, sw.getVoltage());
    
    testVoltage2 = -99;
    assertEquals("up -> +42V", 42, sw.getVoltage());
    
    sw.setDown();
    assertEquals("down -> -99V", -99, sw.getVoltage());
  }
  

  public void test_BetterOne2TwoSwitch() {
    testVoltage1 = 10;
    BetterOne2TwoSwitch sw = new BetterOne2TwoSwitch(testWire1);
    
    IWire upWire = sw.getVoltageUpWire();
    IWire downWire = sw.getVoltageDownWire();
    
    assertEquals("up:  up wire --> 10)", 10, upWire.getVoltage());
    assertEquals("up:  down wire --> 0)", 0, downWire.getVoltage());
    
    sw.setDown();
    assertEquals("down:  up wire --> 0)", 0, upWire.getVoltage());
    assertEquals("down:  down wire --> 10)", 10, downWire.getVoltage());
    
    testVoltage1 = 42;
    assertEquals("down:  up wire --> 0)", 0, upWire.getVoltage());
    assertEquals("down:  down wire --> 42)", 42, downWire.getVoltage());
    
    sw.setUp();
    assertEquals("up:  up wire --> 42)", 42, upWire.getVoltage());
    assertEquals("up:  down wire --> 0)", 0, downWire.getVoltage());
    
  }
  
  
  
  
  public void test_BetterThreeWaySwitch() {
    
    testVoltage1 = 120;
    
    BetterThreeWaySwitch sw = new BetterThreeWaySwitch(testWire1);
    
    assertEquals("up/up --> 120", 120, sw.getVoltage());
    
    sw.setSwitchADown();
    assertEquals("down/up --> 0", 0, sw.getVoltage());
    
    sw.setSwitchAUp();
    assertEquals("up/up --> 120", 120, sw.getVoltage());
    
    sw.setSwitchBDown();
    assertEquals("up/down --> 0", 0, sw.getVoltage());
    
    sw.setSwitchBUp();
    assertEquals("up/up --> 120", 120, sw.getVoltage());
    
    sw.setSwitchADown();
    sw.setSwitchBDown();
    assertEquals("down/down --> 120", 120, sw.getVoltage());
    
    testVoltage1 = -99;
    assertEquals("down/down --> -99", -99, sw.getVoltage());
    
    sw.setSwitchAUp();
    assertEquals("up/down --> 0", 0, sw.getVoltage());
    
    sw.setSwitchBUp();
    assertEquals("up/up --> -99", -99, sw.getVoltage());
    
  }
  
  
  
  
  
}



